import http from './interface'
import {
  platform
} from '../utils/index';
import store from '@/store/index.js'

import {
  goLoginPage
} from "@/mixins/index.js"

const program = {
  H5: "wap",
  APP: "app",
  APPNVUE: "app",
  WEIXIN: "miniProgram"
};
const whiteCodes = [200, 201, 4444]

export const $http = (url, method, data, json, loadingBool = false) => {
  //设置请求前拦截器
  http.interceptor.request = (config) => {
    if (loadingBool && url !== '/homepage/study') {
      uni.showLoading({
        title: '加载中...'
      })
    }

    config.header = {
      "content-type": json ? 'application/json' : 'application/x-www-form-urlencoded',
      // "x-token": uni.getStorageSync('token'),
      "x-token": store.getters.token,
      "Program": program[platform()]
    }
  }
  //设置请求结束后拦截器
  http.interceptor.response = async (response) => {
    //判断返回状态 执行相应操作
    if (loadingBool) {
      uni.hideLoading()
    }
    const res = response.data;
    if (whiteCodes.indexOf(res.code) < 0) {
      if (res.code === 1001) {
        goLoginPage()
        return {}
      }

      uni.showToast({
        title: res.message || 'Error',
        icon: 'none',
        duration: 1500
      })

      return Promise.reject(new Error(res.message || 'Error'))
    } else {
      return res
    }
  }
  return http.request({
    method: method,
    url: url,
    dataType: 'json',
    data,
  })
}

async function login() {
  //返回环宇token所需的login code
  return new Promise(resolve => {
    uni.login({
      provider: 'weixin',
      success(loginRes) {
        resolve(loginRes.code)
      },
      fail() {}
    });
  })
}

async function doRequest(response, url) {
  var code = await login()
  var res = await get('/v1/oauth/refreshToken/code/' + code, {})
  if (res && res.data.data.token) {
    let config = response.config
    uni.setStorageSync("token", res.data.data.token);
    config.header['Authorization'] = res.data.data.token
    let json = config.header["Content-Type"] === 'application/json'
    const resold = await $http(url, config.method, {
      ...config.data
    }, json)
    return resold
  } else {
    uni.clearStorage()
    uni.showToast({
      title: "授权失效，请重新登录",
      duration: 1000,
    })
    uni.navigateTo({
      url: '/pages/login/auth'
    })
    return false
  }
}

function postJson(url, data, loadingBool) {
  return $http(url, 'POST', data, false, loadingBool)
}

function get(url, data, loadingBool) {
  return $http(url, 'GET', data, false, loadingBool)
}

function post(url, data, loadingBool) {
  return $http(url, 'POST', data, true, loadingBool)
}

function put(url, data, loadingBool) {
  return $http(url, 'PUT', data, true, loadingBool)
}

function del(url, data, loadingBool) {
  return $http(url, 'DELETE', data, true, loadingBool)
}

//封装的通用请求方法
export const axios = function(obj = {}) {

  if (typeof obj !== 'object') {
    throw new Error('arguments must be object');
  }
  if (!obj.url) {
    throw new Error('url not defined');
  }
  return new Promise((resolve, reject) => {
    if (obj.isLoading) {
      uni.showLoading({
        title: '加载中',
        mask: true,
      })
    }

    let _url = process.env.VUE_APP_BASE_URL + process.env.VUE_APP_BASE_API + obj.url

    let isGet = obj.method && obj.method.toLowerCase() == 'get';
    let isQuery = obj.isQuery;
    if (isGet || isQuery) {
      _url += '?' + Object.entries(obj.params || {}).map(item => item.join("=")).join("&")
    }
    let _type = 'application/x-www-form-urlencoded';
    if (obj.type == 'json') {
      _type = 'application/json'
    }

    uni.request({
      url: _url,
      data: isGet || isQuery ? '' : (obj.params || ''),
      header: {
        'content-type': _type,
        "x-token": store.getters.token,
      },
      method: obj.method ? obj.method : "POST",
      success: function(ress) {

        console.log(ress, "----------------")

        if (obj.isLoading) {
          uni.hideLoading();
        }

        const res = ress.data;
        if (whiteCodes.indexOf(res.code) < 0) {


          uni.showToast({
            title: res.message || 'Error',
            icon: 'none',
            duration: 1500
          })
          reject(new Error(res.message || 'Error'));
        } else {
          resolve(res)
        }


      },
      fail: function(err) {
        uni.hideLoading();
        uni.showModal({
          title: '提示',
          content: '网络连接失败' + JSON.stringify(err),
        })
        reject(err);
      },
    })
  })
}

export default {
  postJson,
  get,
  post,
  put,
  del
}